#
# See the file "LICENSE" for the full license governing this code.
#
Puppet::Functions.create_function(:'wls_install::artifact_exists') do
  dispatch :artifact_exists do
    param 'String', :domain_dir
    param 'String', :type
    param 'String', :wls_object
    optional_param 'String', :sub_type
    return_type 'Boolean'
  end

  def artifact_exists(domain_dir, type, wls_object, sub_type = '')
    if type == 'resource' || type == 'resource_entry'
      if sub_type.empty?
        return false
      end
    end

    scope = closure_scope
    fact = scope['wls_install_domains']
    if fact.empty?
      call_function('wls_install::log', 'wls_install::artifact_exists wls_install_domains empty, return false')
      return false
    else
      fact.keys.each do |domain|
        if fact[domain]['path'] == domain_dir

          case type
          when 'jdbc'
            jdbc = fact[domain]['jdbc']
            unless jdbc.empty?
              return true if jdbc.include?(wls_object)
            end

          when 'cluster'
            clusters = fact[domain]['clusters']
            unless clusters.empty?
              return true if clusters.include?(wls_object)
            end

          when 'coherence'
            coherence_cluster = fact[domain]['coherence_clusters']
            unless coherence_cluster.empty?
              return true if coherence_cluster.include?(wls_object)
            end

          when 'server'
            servers = fact[domain]['servers']
            unless servers.empty?
              return true if servers.include?(wls_object)
            end

          when 'machine'
            servers = fact[domain]['servers']
            unless servers.empty?
              servers.each do |server|
                server.each do |entry|
                  return true if entry['machine'] == wls_object
                end
              end
            end

          when 'server_templates'
            server_templates = fact[domain]['templates']
            unless server_templates.empty?
              return true if server_templates.include?(wls_object)
            end

          when 'resource'
            adapter = wls_object.downcase
            plan = sub_type.strip

            plan_value = fact[domain]["eis_#{adapter}_plan"]
            unless plan_value.empty?
              return true if plan_value.include?(plan)
            end

          when 'resource_entry'
            adapter = wls_object.downcase
            entry = sub_type.strip

            plan_entries = fact[domain]["eis_#{adapter}_entries"]
            unless plan_entries.empty?
              return true if plan_entries.include?(entry)
            end
    
          when 'deployments'
            deployments = fact[domain]['deployments']
            unless deployments.empty?
              return true if deployments.include?(wls_object)
            end

          when 'filestore'
            filestores = fact[domain]['filestores']
            unless filestores.empty?
              return true if filestores.include?(wls_object)
            end

          when 'jdbcstore'
            jdbcstores = fact[domain]['jdbcstores']
            unless jdbcstores.empty?
              return true if jdbcstores.include?(wls_object)
            end

          when 'safagent'
            safagents = fact[domain]['safagents']
            unless safagents.empty?
              return true if safagents.include?(wls_object)
            end

          when 'jmsserver'
            jmsservers = fact[domain]['jmsservers']
            unless jmsservers.empty?
              return true if jmsservers.include?(wls_object)
            end

          when 'jmsmodule'
            jmsmodules = fact[domain]['jmsmodules']
            unless jmsmodules.empty?
              return true if jmsmodules.include?(wls_object)
            end

          when 'jmsobject'
            jmsmodules = fact[domain]['jmsmodules']
            if jmsmodules.length > 0
              jmsmodules.each do |jmsmodule|
                jmsmodule.each do |entry|
                  jmsobjects = entry['objects']
                  unless jmsobjects.nil?
                    unless jmsobjects.empty?
                      return true if jmsobjects.include?(wls_object)
                    end
                  end
                end
              end
            end

          when 'jmssubdeployment'
            jmsmodules = fact[domain]['jmsmodules']
            if jmsmodules.length > 0
              jmsmodules.each do |jmsmodule|
                jmsmodule.each do |entry|
                  subdeployments = entry['subdeployments']
                  unless subdeployments.nil?
                    unless subdeployments.empty?
                      return true if subdeployments.include?(wls_object)
                    end
                  end
                end
              end
            end

          when 'jmsquota'
            jmsmodules = fact[domain]['jmsmodules']
            if jmsmodules.length > 0
              jmsmodules.each do |jmsmodule|
                jmsmodule.each do |entry|
                  quotas = entry['quotas']
                  unless quotas.nil?
                    unless quotas.empty?
                      return true if quotas.include?(wls_object)
                    end
                  end
                end
              end
            end

          when 'foreignserver'
            jmsmodules = fact[domain]['jmsmodules']
            if jmsmodules.length > 0
              jmsmodules.each do |jmsmodule|
                jmsmodule.each do |entry|
                  foreign_servers = entry['foreign_servers']
                  unless foreign_servers.nil?
                    unless foreign_servers.empty?
                      return true if foreign_servers.include?(wls_object)
                    end
                  end
                end
              end
            end

          when 'foreignserver_object'
            jmsmodules = fact[domain]['jmsmodules']
            if jmsmodules.length > 0
              jmsmodules.each do |jmsmodule|
                jmsmodule.each do |entry|
                  foreign_servers = entry['foreign_servers']
                  unless foreign_servers.nil?
                    unless foreign_servers.empty?
                      foreign_servers.each do |value|
                        foreign_server_objects = value['objects']
                        unless foreign_server_objects.nil?
                          unless foreign_server_objects.empty?
                            return true if foreign_server_objects.include?(wls_object)
                          end
                        end
                      end
                    end
                  end
                end
              end
            end

          end
        end
      end
      return false
    end
  end
end